import 'package:chopper/chopper.dart';
import 'package:chopper_client_wrapper/ChopperService/Utils/Convertors/JsonToTypeConverter.dart';
import 'package:chopper_client_wrapper/ChopperService/Utils/Interceptors/ApplyHeaderInterceptor.dart';
import 'package:chopper_client_wrapper/ChopperService/Utils/Interceptors/RequestLogger.dart';
import 'package:chopper_client_wrapper/ChopperService/Utils/Interceptors/ResponseLogger.dart';

abstract class ChopperClientWrapper {
  late ChopperClient _client;
  ChopperClientWrapper() {
    _createChopperClient();
  }
  // Json to model converter stores a Map that has a type and it's method that convertes json to its type, for e.g. LoginResponseModel: LoginResponseModel.fromJson
  Map<Type, JsonConvertorFunction> get jsonToModelConvertorMap;
  // Json to model converter stores a Map that has a type and it's method that convertes json to its type, for e.g. ErrorResponseModel: ErrorResponseModel.fromJson
  Map<Type, JsonConvertorFunction> get jsonToErrorModelConvertorMap;

  // Base url of your api service
  String get baseUrl;

  // multiple chopper services that you create for you appication should be assigned here
  List<ChopperService> get chopperServices;

  // this method has to be override compulsory to get token that can be passed to header
  String getToken();

  // some other header prameters that you might want to add header
  Map<String, String> getHeaderParameters() {
    return {};
  }

  // using this method you can access any particular chopper service that you have created and assigned in chopperServices
  T getChopperService<T extends ChopperService>() {
    return _client.getService<T>();
  }

  void _createChopperClient() {
    _client = ChopperClient(
        baseUrl: Uri.parse(baseUrl),
        services: chopperServices,
        interceptors: [
          // logs request before sending to server. It logs request type, url, and body
          RequestLogger(),
          // logs response that came from server and also logs error if there
          ResponseLogger(),
          // header inceptor that adds header before sending to server
          ApplyHeaderInterceptor(
              getHeaderToken: getToken, getHeaderValues: getHeaderParameters),
        ],
        // converts JSON to model
        converter:
            JsonToTypeConverter(jsonConvertorMap: jsonToModelConvertorMap),
        errorConverter: JsonToTypeConverter(
            jsonConvertorMap: jsonToErrorModelConvertorMap));
  }
}
